@@ -163,7 +163,7 @@ static void sbr_dequant(SpectralBandReplication *sbr, int id_aac)
             for (k = 0; k < sbr->n[sbr->data[0].bs_freq_res[e]]; k++) {
                 SoftFloat temp1, temp2, fac;
 
-                temp1.exp = sbr->data[0].env_facs[e][k].mant * alpha + 14;
+                temp1.exp = sbr->data[0].env_facs_q[e][k] * alpha + 14;
                 if (temp1.exp & 1)
                   temp1.mant = 759250125;
                 else
@@ -174,7 +174,7 @@ static void sbr_dequant(SpectralBandReplication *sbr, int id_aac)
                     temp1 = FLOAT_1;
                 }
 
-                temp2.exp = (pan_offset - sbr->data[1].env_facs[e][k].mant) * alpha;
+                temp2.exp = (pan_offset - sbr->data[1].env_facs_q[e][k]) * alpha;
                 if (temp2.exp & 1)
                   temp2.mant = 759250125;
                 else
@@ -190,13 +190,10 @@ static void sbr_dequant(SpectralBandReplication *sbr, int id_aac)
                 SoftFloat temp1, temp2, fac;
 
                 temp1.exp = NOISE_FLOOR_OFFSET - \
-                    sbr->data[0].noise_facs[e][k].mant + 2;
+                    sbr->data[0].noise_facs_q[e][k] + 2;
                 temp1.mant = 0x20000000;
-                if (temp1.exp > 66) { // temp1 > 1E20
-                    av_log(NULL, AV_LOG_ERROR, "envelope scalefactor overflow in dequant\n");
-                    temp1 = FLOAT_1;
-                }
-                temp2.exp = 12 - sbr->data[1].noise_facs[e][k].mant + 1;
+                av_assert0(temp1.exp <= 66);
+                temp2.exp = 12 - sbr->data[1].noise_facs_q[e][k] + 1;
                 temp2.mant = 0x20000000;
                 fac   = av_div_sf(temp1, av_add_sf(FLOAT_1, temp2));
                 sbr->data[0].noise_facs[e][k] = fac;
@@ -210,7 +207,7 @@ static void sbr_dequant(SpectralBandReplication *sbr, int id_aac)
                 for (k = 0; k < sbr->n[sbr->data[ch].bs_freq_res[e]]; k++){
                     SoftFloat temp1;
 
-                    temp1.exp = alpha * sbr->data[ch].env_facs[e][k].mant + 12;
+                    temp1.exp = alpha * sbr->data[ch].env_facs_q[e][k] + 12;
                     if (temp1.exp & 1)
                         temp1.mant = 759250125;
                     else
@@ -225,7 +222,7 @@ static void sbr_dequant(SpectralBandReplication *sbr, int id_aac)
             for (e = 1; e <= sbr->data[ch].bs_num_noise; e++)
                 for (k = 0; k < sbr->n_q; k++){
                     sbr->data[ch].noise_facs[e][k].exp = NOISE_FLOOR_OFFSET - \
-                        sbr->data[ch].noise_facs[e][k].mant + 1;
+                        sbr->data[ch].noise_facs_q[e][k] + 1;
                     sbr->data[ch].noise_facs[e][k].mant = 0x20000000;
                 }
         }
@@ -440,7 +437,6 @@ static void sbr_gain_calc(AACContext *ac, SpectralBandReplication *sbr,
                                                 av_add_sf(FLOAT_1, sbr->e_curr[e][m]),
                                                 av_add_sf(FLOAT_1, sbr->q_mapped[e][m]))));
                 }
-                sbr->gain[e][m] = av_add_sf(sbr->gain[e][m], FLOAT_MIN);
             }
             for (m = sbr->f_tablelim[k] - sbr->kx[1]; m < sbr->f_tablelim[k + 1] - sbr->kx[1]; m++) {
                 sum[0] = av_add_sf(sum[0], sbr->e_origmapped[e][m]);
@@ -575,8 +571,7 @@ static void sbr_hf_assemble(int Y1[38][64][2],
                 int A = (1-((indexsine+(kx & 1))&2));
                 int B = (A^(-idx)) + idx;
                 int *out = &Y1[i][kx][idx];
-                int shift;
-                unsigned round;
+                int shift, round;
 
                 SoftFloat *in  = sbr->s_m[e];
                 for (m = 0; m+1 < m_max; m+=2) {
@@ -589,12 +584,12 @@ static void sbr_hf_assemble(int Y1[38][64][2],
                     }
                     if (shift < 32) {
                         round = 1 << (shift-1);
-                        out[2*m  ] += (int)(in[m  ].mant * A + round) >> shift;
+                        out[2*m  ] += (in[m  ].mant * A + round) >> shift;
                     }
 
                     if (shift2 < 32) {
                         round = 1 << (shift2-1);
-                        out[2*m+2] += (int)(in[m+1].mant * B + round) >> shift2;
+                        out[2*m+2] += (in[m+1].mant * B + round) >> shift2;
                     }
                 }
                 if(m_max&1)
@@ -605,7 +600,7 @@ static void sbr_hf_assemble(int Y1[38][64][2],
                         return;
                     } else if (shift < 32) {
                         round = 1 << (shift-1);
-                        out[2*m  ] += (int)(in[m  ].mant * A + round) >> shift;
+                        out[2*m  ] += (in[m  ].mant * A + round) >> shift;
                     }
                 }
             }
